#!/usr/bin/env python3

import argparse
from utils import utils
from iseqresources.update_record import UpdateRecord
import json
import os


__version__ = '0.0.10'


GITLAB_TOKEN=os.environ.get("GITLAB_TOKEN")


def info_text():
    utils.clear_screen()
    print('''Press 0 to exit
Press 1 to update a tool/database''')
    return int(input('Enter a number of your choice: '))


def tools_in_json(tools_and_databases: list):
    names = []
    for item in tools_and_databases:
        names.append(item['name'])
    return names


def update_tool_record(gitlab_token, json_file="https://gitlab.com/intelliseq/iseqresources/-/raw/main/json/tools_and_databases.json"):
    input_dict = utils.load_json(json_file)
    resources_dict = utils.load_json(json_file)
    update_expected_versions = {
        "github": False,
        "url-check": True,
        "update-every-nth-month": False
    }
    choice = 1
    while choice != 0:
        print("Tools and databases: " + ', '.join(tools_in_json(resources_dict)))
        name_to_update = input("Enter name of tool/database to update: ")
        tool_found = False
        for tool_or_database in resources_dict:
            if tool_or_database['name'] == name_to_update:
                obj = UpdateRecord(tool_or_database, update_expected_versions.get(tool_or_database["test"], False))
                obj.update_record()
                print(f"Tool/database {name_to_update} updated and now look like this:")
                print(json.dumps(tool_or_database, indent=2))
                input("Press enter to continue")
                tool_found = True
                break
        if not tool_found:
            print("Tool/database not found")
            input("Press enter to continue")
        choice = info_text()
    if (input_dict != resources_dict):
        if json_file.startswith("https://"):
            if not gitlab_token:
                gitlab_token = utils.get_gitlab_token()
            utils.save_json_to_gitlab(resources_dict, gitlab_token)
        else:
            utils.save_json(json_file, resources_dict)


def main():
    parser = argparse.ArgumentParser(description='Add new tool to json file')
    parser.add_argument('--input-json', type=str, required=False,
                        help='Json file to which to update a new field')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {}'.format(__version__))
    args = parser.parse_args()

    if args.input_json:
        update_tool_record(gitlab_token=GITLAB_TOKEN,
            json_file=args.input_json)
    else:
        update_tool_record(gitlab_token=GITLAB_TOKEN)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
from iseqresources.add_tool import AddTool
from utils import utils
import os


__version__ = '0.0.10'


GITLAB_TOKEN=os.environ.get("GITLAB_TOKEN")


def info_text():
    return '''Press 0 to exit
Press 1 to add a tool from github
Press 2 to add a tool from website
Press 3 to add a tool from website without specific released version'''


def add_tool_or_database(gitlab_token, json_file="https://gitlab.com/intelliseq/iseqresources/-/raw/main/json/tools_and_databases.json"):
    if not gitlab_token and json_file.startswith("https://"):
        gitlab_token = utils.get_gitlab_token()
    obj = AddTool(json_file, gitlab_token)
    switcher={
        0: lambda : obj.exit(),
        1: lambda : obj.add_github_tool(),
        2: lambda : obj.add_website_tool_with_released_version(),
        3: lambda : obj.add_website_tool_without_released_version()
    }
    choice = 1
    while choice != 0:
        print(info_text())
        choice = int(input('Enter a number of your choice: '))
        switcher.get(choice, lambda : "ERROR: Invalid Operation")()


def main():
    parser = argparse.ArgumentParser(description='Add new tool to json file')
    parser.add_argument('--input-json', type=str, required=False,
                        help='Json file to which to enter a new field')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {}'.format(__version__))
    args = parser.parse_args()

    if args.input_json:
        add_tool_or_database(gitlab_token=GITLAB_TOKEN,
            json_file=args.input_json)
    else:    
        add_tool_or_database(gitlab_token=GITLAB_TOKEN)


if __name__ == "__main__":
    main()



from utils import utils
import sys
import datetime
import json
import requests

class AddTool:

    def __init__(self, json_file: str, gitlab_token: str):
        self.json_file = json_file
        self.resources_dict = utils.load_json(json_file)
        self.gitlab_token = gitlab_token

    def add_name(self) -> str:
        return input('Enter a tool/database name: ')

    def add_current_version(self, name: str) -> str:
        format = {
            "github": "(e.g. v1.0.0)",
            "url-check": "(e.g. for Ensembl: 107)",
            "update-every-nth-month": "(date in format YYYY/MM/DD)"
        }
        return input(f'Enter a tool/database current version {format.get(name, None)}: ')

    def add_update_task(self) -> list:
        update_task = input('Enter the names of the tasks that should be updated after the new version of the tool (format: task_name_1, task_name_2): ')
        return update_task.replace(" ", "").split(",")

    def add_github_repo(self) -> str:
        github_repo = input('Enter a tool/database repo in github (e.g. https://github.com/lgmgeo/AnnotSV): ')
        if "github.com" in github_repo:
            if self.check_github_repo(github_repo):
                return github_repo.split("github.com/")[1].split("/")[0] + "/" + github_repo.split("github.com/")[1].split("/")[1]
        else:
            print("Invalid repo url")
            return self.add_github_repo()
    
    def check_github_repo(self, repo: str) -> str:
        response = requests.get(repo)
        if response.ok:
            return repo
        else:
            print("Repo does not exist")
            return self.add_github_repo()
   
    def add_expected_version(self) -> list:
        expected_version = input('Enter a tool/database expected versions (format: expected_version_1, expected_version_2): ')
        return expected_version.replace(" ", "").split(",")

    def add_release_day(self) -> str:
        release_day = {
            "NO": "unknown_released",
            "YES": "known_released"
        }
        tool_released_day = input('Does the tool have a known release day (e.g. Clinvar has not https://ftp.ncbi.nlm.nih.gov/pub/clinvar/vcf_GRCh38/)? YES or NO? ')
        return release_day.get(tool_released_day.upper(), None)

    def add_url(self, name: str) -> str:
        text = {
            "url-check_known_released": "Enter a tool/database url and specify where in url expected_version is (e.g. http://ftp.ensembl.org/pub/release-{expected_version}/): ",
            "url-check_unknown_released": "Enter a tool/database url and specify where in url expected_version and release_day is (e.g. https://ftp.ncbi.nlm.nih.gov/pub/clinvar/vcf_GRCh38/clinvar_{expected_version}{release_day}.vcf.gz.md5): ",
            "update-every-nth-month": "Enter (optionally) a tool/database url (e.g. https://civicdb.org/releases): "
        }
        if name == "update-every-nth-month":
            return input(f'{text.get(name, None)}')
        return self.check_url_str(input(text.get(name)), name)
    
    def check_url_str(self, url: str, name: str):
        expected_structure = {
            "url-check_known_released": [
                "{expected_version}"
                ],
            "url-check_unknown_released": [
                "{expected_version}",
                "{release_day}"
                ]
        }
        check = [structure in url for structure in expected_structure.get(name, None)]
        if all(item for item in check):
            return url
        else:
            print("Invalid url")
            return self.add_url(name)

    def add_update_every_nth_month(self) -> int:
        return int(input('Enter every how many months it should be updated: '))

    def validate_date(self, date: str):
        try:
            datetime.datetime.strptime(date, '%Y/%m/%d')
            return date
        except ValueError:
            print("Incorrect data format, should be YYYY/MM/DD")
            return self.validate_date(self.add_current_version(name="website_without_released"))

    def add_github_tool(self):
        tool_or_database={
            "name": self.add_name(),
            "current_version": self.add_current_version(name="github"),
            "newest_version": "",
            "last_check": "",
            "test": "github",
            "repoWithOwner": self.add_github_repo(),
            "update_task": self.add_update_task()
        }
        self.ask_about_changes(tool_or_database)
        self.resources_dict.append(tool_or_database)
        if self.gitlab_token:
            utils.save_json_to_gitlab(self.resources_dict, self.gitlab_token)
        else:
            utils.save_json(self.json_file, self.resources_dict)

    def add_website_tool_with_released_version(self):
        known_release_day = self.add_release_day()
        tool_or_database={
            "name": self.add_name(),
            "current_version": self.add_current_version(name="url-check"),
            "expected_version": self.add_expected_version(),
            "known_release_day": known_release_day,
            "newest_version": "",
            "last_check": "",
            "test": "url-check",
            "url": self.add_url(name="url-check_unknown_released") if known_release_day == "unknown_released" else self.add_url(name="url-check_known_released"),
            "update_task": self.add_update_task()
        }
        self.ask_about_changes(tool_or_database)
        self.resources_dict.append(tool_or_database)
        if self.gitlab_token:
            utils.save_json_to_gitlab(self.resources_dict, self.gitlab_token)
        else:
            utils.save_json(self.json_file, self.resources_dict)

    def add_website_tool_without_released_version(self):
        tool_or_database={
            "name": self.add_name(),
            "current_version": self.validate_date(self.add_current_version(name="update-every-nth-month")),
            "newest_version": "",
            "update_every_nth_month": self.add_update_every_nth_month(),
            "test": "update-every-nth-month",
            "url": self.add_url(name="update-every-nth-month"),
            "update_task": self.add_update_task()
        }
        self.ask_about_changes(tool_or_database)
        self.resources_dict.append(tool_or_database)
        if self.gitlab_token:
            utils.save_json_to_gitlab(self.resources_dict, self.gitlab_token)
        else:
            utils.save_json(self.json_file, self.resources_dict)
    
    def ask_about_changes(self, added_tool: dict):
        utils.clear_screen()
        print('''Your tool/database look like this:''')
        print(json.dumps(added_tool, indent=2))
        changes = input('Do you want to make changes? YES or NO? ')
        if changes.upper() == "YES":
            added_tool = self.make_changes(added_tool)
            self.ask_about_changes(added_tool)
    
    def make_changes(self, added_tool: dict):
        field_change = input('Which field you want to change? ')
        known_released = added_tool.get("known_release_day", None)
        url_test = added_tool["test"] + f"_{known_released}" if known_released else added_tool["test"]
        all_fields={
            "name": lambda : self.make_change_in_name(),
            "current_version": lambda : self.make_change_in_current_version(name=added_tool["test"]),
            "expected_version": lambda : self.make_change_in_expected_version(),
            "update_every_nth_month": lambda : self.make_change_in_update_every_nth_month(),
            "known_release_day": lambda : self.make_change_in_known_release_day(),
            "repoWithOwner": lambda : self.make_change_in_repo(),
            "url": lambda : self.make_change_in_url(name=url_test),
            "update_task": lambda : self.make_change_in_update_task()
        }
        added_tool[field_change] = all_fields.get(field_change, lambda : "ERROR: Invalid input")()
        return added_tool
    
    def make_change_in_name(self):
        return self.add_name()
    
    def make_change_in_current_version(self, name: str):
        if name == "update-every-nth-month":
            return self.validate_date(self.add_current_version(name=name))
        return self.add_current_version(name=name)

    def make_change_in_expected_version(self):
        return self.add_expected_version()

    def make_change_in_update_every_nth_month(self):
        return self.add_update_every_nth_month()
    
    def make_change_in_known_release_day(self):
        return self.add_release_day()

    def make_change_in_repo(self):
        return self.add_github_repo()

    def make_change_in_url(self, name: str):
        return self.add_url(name=name)

    def make_change_in_update_task(self):
        return self.add_update_task()        

    def exit(self):
        sys.exit()



import requests
from packaging import version
from datetime import date, datetime
from dateutil.relativedelta import relativedelta
from itertools import dropwhile


class CheckVersion:

    def __init__(self, tool: dict, token: str):
        self.tool = tool
        self.token = token
 
    def get_todays_date(self) -> date:
        today = date.today()
        return today.strftime("%Y/%m/%d")

    def github_auth(self) -> dict:
        return {'Authorization': f'token {self.token}'}
   
    def check_github_repo(self) -> bool:
        currentVersion = version.parse(self.tool["current_version"])
        repoWithOwner= self.tool["repoWithOwner"]
        response = requests.get(f"https://api.github.com/repos/{repoWithOwner}/releases", headers=self.github_auth())
        links = [t["html_url"] for t in response.json() if version.parse(t["tag_name"]) > currentVersion]
        releases = [t["tag_name"] for t in response.json() if version.parse(t["tag_name"]) > currentVersion]
        self.tool["newest_version"] = releases[0] + f" ({links[0]})" if releases else ""
        self.tool["last_check"] = self.get_todays_date()
        return True if releases else False
   
    def check_url_without_released_version(self) -> bool:
        url = self.tool["url"]
        currentVersion = datetime.strptime(self.tool["current_version"], '%Y/%m/%d')
        currentVersion_plus_nth_months = currentVersion + relativedelta(months=+self.tool["update_every_nth_month"])
        currentVersion_plus_nth_months = currentVersion_plus_nth_months.strftime("%Y/%m/%d")
        if self.get_todays_date() > currentVersion_plus_nth_months:
            self.tool["newest_version"] = self.get_todays_date() + f" ({url})"
            return True
        return False

    def check_url_with_released_version(self) -> bool:
        newest_version = ""
        self.tool["last_check"] = self.get_todays_date()
        if self.tool["known_release_day"] == "NO":
            release_days = ["%.2d" % i for i in range(32)]
            for expected_version in self.tool["expected_version"]:
                for release_day in release_days:
                    url = self.tool["url"].format(expected_version=expected_version, release_day=release_day)
                    response = requests.get(url)
                    if response.ok:
                        newest_version = expected_version
                        self.tool["newest_version"] = expected_version + f" ({url})"
                        break
        else:
            for expected_version in self.tool["expected_version"]:
                url = self.tool["url"].format(expected_version=expected_version)
                response = requests.get(url)
                if response.ok:
                    newest_version = expected_version
                    self.tool["newest_version"] = expected_version + f" ({url})"
        if newest_version:
            self.tool["expected_version"] = list(dropwhile(lambda x: x != newest_version, self.tool["expected_version"]))
            return True
        return False



#!/usr/bin/env python3

import argparse
from tqdm import tqdm
from utils import utils
from iseqresources.check_version import CheckVersion
from iseqresources.add_task_to_jira import AddTaskToJira
import os


__version__ = '0.0.10'


GITHUB_TOKEN=os.environ.get("GITHUB_TOKEN")
GITLAB_TOKEN=os.environ.get("GITLAB_TOKEN")
JIRA_EMAIL=os.environ.get("JIRA_EMAIL")
JIRA_TOKEN=os.environ.get("JIRA_TOKEN")


def check_for_new_version(github_token, gitlab_token, jira_email, jira_token,
                        json_file="https://gitlab.com/intelliseq/iseqresources/-/raw/main/json/tools_and_databases.json", 
                        info_json="https://gitlab.com/intelliseq/iseqresources/-/raw/main/json/info.json"):
    resources_dict = utils.load_json(json_file)
    jira_project_info = utils.load_json(info_json)
    if not github_token:
        github_token = utils.get_github_token()
    if not jira_email:
        jira_email, jira_token = utils.get_jira_auth(server=jira_project_info['server'], epic_id=jira_project_info['epic_id'])
    test_name = {
        "github": lambda : obj.check_github_repo(),
        "url-check": lambda : obj.check_url_with_released_version(),
        "update-every-nth-month": lambda : obj.check_url_without_released_version()
    }
    for tool_or_database in tqdm(resources_dict):
        obj = CheckVersion(tool_or_database, github_token)
        create_task_in_jira = test_name.get(tool_or_database["test"], lambda : "ERROR: Invalid test")()
        # create task in jira if there is new version of tool/database
        if create_task_in_jira:
            jira = AddTaskToJira(tool_or_database, jira_email, jira_token, jira_project_info=jira_project_info)
            jira.add_task_to_jira()
    if json_file.startswith("https://"):
        if not gitlab_token:
            gitlab_token = utils.get_gitlab_token()
        utils.save_json_to_gitlab(resources_dict, gitlab_token)
    else:
        utils.save_json(json_file, resources_dict)
    return create_task_in_jira


def main():
    parser = argparse.ArgumentParser(description='')
    parser.add_argument('--input-json', type=str, required=False,
                        help='Json file to which to enter a new field')
    parser.add_argument('--info-json', type=str, required=False,
                        help='Json file with info about JIRA project (server, epic_id and project_id)')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {}'.format(__version__))
    args = parser.parse_args()
    
    if args.input_json:
        check_for_new_version(github_token=GITHUB_TOKEN,
            gitlab_token=GITLAB_TOKEN,
            jira_email=JIRA_EMAIL,
            jira_token=JIRA_TOKEN,
            json_file=args.input_json, 
            info_json=args.info_json)
    else:
        check_for_new_version(github_token=GITHUB_TOKEN,
            gitlab_token=GITLAB_TOKEN,
            jira_email=JIRA_EMAIL,
            jira_token=JIRA_TOKEN)


if __name__ == "__main__":
    main()



from jira import JIRA
from utils import utils


class AddTaskToJira:
    
    def __init__(self, tool: dict, email: str, token: str, jira_project_info: dict):
        self.tool = tool
        self.email = email
        self.token = token
        self.server = jira_project_info["server"]
        self.epic_id = jira_project_info["epic_id"]
        self.project_key = jira_project_info["project_key"]
        self.jira_connection = self.jira_auth()
        self.epic = self.connect_epic()

    def jira_auth(self):
        return JIRA(basic_auth=(self.email, self.token), server=self.server)

    def connect_epic(self):
        return self.jira_connection.issue(self.epic_id)

    def create_issue_dict(self):
        issue_dict = {
            'project': {'key': self.project_key},
            'summary': f'Aktualizacja {self.tool["name"]}',
            'description': 
            f'Obecna wersja narzędzia {self.tool["name"]}: {self.tool["current_version"]}; Najnowsza wersja narzędzia: {self.tool["newest_version"]}',
            'issuetype': {'name': 'Task'}
        }
        return issue_dict

    def create_issue_dict_subtask(self):
        tasks = ", ".join(self.tool["update_task"])
        issue_dict_subtask = {
            'project': {'key': self.project_key},
            'summary': f'Aktualizacja tasków po aktualizacji narzędzia {self.tool["name"]}',
            'description': f'Należy zaktualizować taski: {tasks}',
            'issuetype': {'name': 'Task'}
        }
        return issue_dict_subtask
    
    def create_issue_jira(self, task_or_subtask: str):
        task_or_subtask_dict = {
            "task": lambda : self.create_issue_dict(),
            "subtask": lambda : self.create_issue_dict_subtask()
        }
        return self.jira_connection.create_issue(fields=task_or_subtask_dict.get(task_or_subtask, lambda : "ERROR: Invalid Operation")())

    def add_task_to_jira(self):
        # task
        new_issue = self.create_issue_jira("task")
        new_issue.update(fields={'parent': {'id': self.epic.id}})
        # subtask
        new_issue_subtask = self.create_issue_jira("subtask")
        self.jira_connection.create_issue_link(type='is blocked by', inwardIssue=new_issue_subtask.key, outwardIssue=new_issue.key)



import json
from getpass import getpass
import urllib.request
import gitlab
import os
import requests
from jira import JIRA


def load_json(json_path: str) -> dict:
    if json_path.startswith("https://"):
        with urllib.request.urlopen(json_path) as url:
            return json.loads(url.read().decode())
    with open(json_path, "r") as json_file:
        return json.load(json_file)


def save_json(json_path: str, resources_dict: dict):
    with open(json_path, "w") as json_file:
        json.dump(resources_dict, json_file, indent=2)


def save_json_to_gitlab(data: dict, gitlab_token: str):
    gl = gitlab.Gitlab(private_token=gitlab_token)
    gl.auth()
    project_id = 38164378 # iseqresources Gitlab ID
    project = gl.projects.get(project_id)
    f = project.files.get(file_path='json/tools_and_databases.json', ref='main')
    f.content = json.dumps(data)
    f.save(branch='main', commit_message='Update file')


def get_gitlab_token():
    print('''Please enter Gitlab token (https://docs.gitlab.com/ee/user/profile/personal_access_tokens.html)''')
    return check_gitlab_token(getpass())


def check_gitlab_token(token: str):
    gl = gitlab.Gitlab(private_token=token)
    try:
        gl.auth()
    except gitlab.GitlabAuthenticationError:
        print("Gitlab token is invalid")
        return get_gitlab_token()
    return token


def get_github_token():
    print('''Please enter Github token (https://docs.github.com/en/authentication/keeping-your-account-and-data-secure/creating-a-personal-access-token)''')
    return check_github_token(getpass())


def check_github_token(token: str):
    headers = {'Authorization': 'token ' + token}
    login = requests.get('https://api.github.com/user', headers=headers)
    if not login.ok:
        print("Github token is invalid")
        return get_github_token()
    return token


def get_jira_auth(server: str, epic_id: str):
    email = input('Please enter JIRA email: ')
    print('''Please enter JIRA token (https://support.atlassian.com/atlassian-account/docs/manage-api-tokens-for-your-atlassian-account/)''')
    token = getpass()
    return check_jira_auth(email, token, server, epic_id)


def check_jira_auth(email: str, token: str, server: str, epic_id: str):
    jira_options = {'server': server}
    try:
        jira = JIRA(options=jira_options, basic_auth=(email, token))
        jira.issue(epic_id)
        return email, token
    except:
        print("JIRA token is invalid")
        return get_jira_auth(server, epic_id)


def clear_screen():
    return os.system('cls' if os.name=='nt' else 'clear')


class UpdateRecord:

    def __init__(self, tool: str, update_expected_versions: bool):
        self.tool = tool
        self.update_expected_versions = update_expected_versions
    
    def current_expected_version(self):
        return self.tool["expected_version"]

    def extend_expected_versions(self):
        print(f'Current expected versions: {self.current_expected_version()}')
        new_expected_versions = input('Enter new expected versions (format: expected_version_1, expected_version_2): ')
        new_expected_versions = new_expected_versions.replace(" ", "").split(",")
        self.tool["expected_version"].extend(new_expected_versions)

    def update_current_version(self):
        if self.tool["newest_version"]:
            self.tool["current_version"] = self.tool["newest_version"].split(" (")[0]

    def clear_newest_version(self):
        self.tool["newest_version"] = ""

    def update_record(self):
        self.update_current_version()
        self.clear_newest_version()
        if self.update_expected_versions:
            self.extend_expected_versions()
